@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -376,7 +376,8 @@ ui_read_column(struct convolveparams *p, int i0k1)
 
   /* Read the desired column(s). */
   out=gal_table_read(filename, hdu, lines, column, p->cp.searchin,
-                     p->cp.ignorecase, p->cp.minmapsize, NULL);
+                     p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
+                     NULL);
   gal_list_str_free(lines, 1);
 
   /* Confirm if only one column was read (it is possible to match more than
@@ -384,7 +385,7 @@ ui_read_column(struct convolveparams *p, int i0k1)
   if(out->next!=NULL)
     {
       if(filename)
-        gal_checkset_dataset_name(filename, hdu);
+        source=gal_checkset_dataset_name(filename, hdu);
       else
         source="standard-input";
       error(EXIT_FAILURE, 0, "%s: more than one column in input table mached "
@@ -435,9 +436,13 @@ ui_read_input(struct convolveparams *p)
       {
         p->input=gal_array_read_one_ch_to_type(p->filename, p->cp.hdu, NULL,
                                                INPUT_USE_TYPE,
-                                               p->cp.minmapsize);
+                                               p->cp.minmapsize,
+                                               p->cp.quietmmap);
         p->input->wcs=gal_wcs_read(p->filename, p->cp.hdu, 0, 0,
                                    &p->input->nwcs);
+        p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                                  p->input->dsize,
+                                                  p->input->wcs);
       }
 
   /* The input isn't an image (wasn't read yet), so we'll read it as a
@@ -459,9 +464,15 @@ ui_read_kernel(struct convolveparams *p)
   if( p->kernelname
       && p->input->ndim>1
       && gal_array_name_recognized(p->kernelname)  )
-    p->kernel = gal_array_read_one_ch_to_type(p->kernelname, p->khdu,
-                                              NULL, INPUT_USE_TYPE,
-                                              p->cp.minmapsize);
+    {
+      p->kernel = gal_array_read_one_ch_to_type(p->kernelname, p->khdu,
+                                                NULL, INPUT_USE_TYPE,
+                                                p->cp.minmapsize,
+                                                p->cp.quietmmap);
+      p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
+                                                 p->kernel->dsize,
+                                                 p->kernel->wcs);
+    }
   else
     p->kernel=ui_read_column(p, 1);
 
@@ -559,11 +570,11 @@ ui_preparations(struct convolveparams *p)
              meaningful. */
           sum=gal_statistics_sum(p->input);
           sum=gal_data_copy_to_new_type_free(sum, GAL_TYPE_FLOAT32);
-          p->input = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE,
+          p->input = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE, 1,
                                     GAL_ARITHMETIC_FLAGS_ALL, p->input, sum);
           sum=gal_statistics_sum(p->kernel);
           sum=gal_data_copy_to_new_type_free(sum, GAL_TYPE_FLOAT32);
-          p->kernel = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE,
+          p->kernel = gal_arithmetic(GAL_ARITHMETIC_OP_DIVIDE, 1,
                                      GAL_ARITHMETIC_FLAGS_ALL, p->kernel, sum);
         }
     }
