@@ -3,9 +3,9 @@ Crop - Crop a given size from one or multiple images.
 Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -123,20 +123,21 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Crop by center",
-      ARGS_GROUP_CENTER_GENERAL
+      UI_GROUP_CENTER_GENERAL
     },
     {
       "checkcenter",
       UI_KEY_CHECKCENTER,
-      "INT",
+      "FLT/INT",
       0,
       "Width (in pixels) of box at center to check.",
-      ARGS_GROUP_CENTER_GENERAL,
-      &p->checkcenter,
-      GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_0_OR_ODD,
+      UI_GROUP_CENTER_GENERAL,
+      &p->incheckcenter,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_float64
     },
     {
       "width",
@@ -144,13 +145,13 @@ struct argp_option program_options[] =
       "FLT[,...]",
       0,
       "Width when crop is defined by its center.",
-      ARGS_GROUP_CENTER_GENERAL,
+      UI_GROUP_CENTER_GENERAL,
       &p->width,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET,
-      ui_parse_width_and_center
+      gal_options_parse_csv_float64
     },
     {
       "center",
@@ -158,13 +159,13 @@ struct argp_option program_options[] =
       "FLT[,...]",
       0,
       "Central coordinates of a single crop.",
-      ARGS_GROUP_CENTER_GENERAL,
+      UI_GROUP_CENTER_GENERAL,
       &p->center,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET,
-      ui_parse_width_and_center
+      gal_options_parse_csv_float64
     },
 
 
@@ -176,7 +177,7 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Crop by center (when a catalog is given)",
-      ARGS_GROUP_CENTER_CATALOG
+      UI_GROUP_CENTER_CATALOG
     },
     {
       "catalog",
@@ -184,7 +185,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Input catalog filename.",
-      ARGS_GROUP_CENTER_CATALOG,
+      UI_GROUP_CENTER_CATALOG,
       &p->catname,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -197,7 +198,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "HDU of catalog, if it is a FITS table.",
-      ARGS_GROUP_CENTER_CATALOG,
+      UI_GROUP_CENTER_CATALOG,
       &p->cathdu,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -210,7 +211,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Column no./info of crop filename (no suffix).",
-      ARGS_GROUP_CENTER_CATALOG,
+      UI_GROUP_CENTER_CATALOG,
       &p->namecol,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -223,7 +224,7 @@ struct argp_option program_options[] =
       "STR/INT",
       0,
       "Column no./info containing coordinates.",
-      ARGS_GROUP_CENTER_CATALOG,
+      UI_GROUP_CENTER_CATALOG,
       &p->coordcol,
       GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
@@ -238,7 +239,7 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Crop by region",
-      ARGS_GROUP_REGION
+      UI_GROUP_REGION
     },
     {
       "section",
@@ -246,7 +247,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Image section string specifying crop range.",
-      ARGS_GROUP_REGION,
+      UI_GROUP_REGION,
       &p->section,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -259,7 +260,7 @@ struct argp_option program_options[] =
       "STR",
       0,
       "Polygon vertices of region to crop, keep inside.",
-      ARGS_GROUP_REGION,
+      UI_GROUP_REGION,
       &p->polygon,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -272,7 +273,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Keep the polygon's outside, mask the inside.",
-      ARGS_GROUP_REGION,
+      UI_GROUP_REGION,
       &p->outpolygon,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
