@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -1013,7 +1013,7 @@ struct argp_option program_options[] =
       UI_KEY_SKY,
       0,
       0,
-      "Average Sky value under this clump or object.",
+      "Sky value (per pixel).",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1027,7 +1027,7 @@ struct argp_option program_options[] =
       UI_KEY_STD,
       0,
       0,
-      "Average Sky standard deviation under label.",
+      "Sky standard deviation (per pixel).",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1121,7 +1121,7 @@ struct argp_option program_options[] =
       UI_KEY_SEMIMAJOR,
       0,
       0,
-      "Flux weighted semi-major axis.",
+      "RMS along major axis (in pixels).",
       UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
@@ -1135,7 +1135,7 @@ struct argp_option program_options[] =
       UI_KEY_SEMIMINOR,
       0,
       0,
-      "Flux weighted semi-minor axis.",
+      "RMS along minor axis (in pixels).",
       UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
@@ -1177,7 +1177,7 @@ struct argp_option program_options[] =
       UI_KEY_GEOSEMIMAJOR,
       0,
       0,
-      "Geometric semi-major axis.",
+      "RMS along major axis (ignoring value).",
       UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
@@ -1191,7 +1191,7 @@ struct argp_option program_options[] =
       UI_KEY_GEOSEMIMINOR,
       0,
       0,
-      "Geometric semi-minor axis.",
+      "RMS along minor axis (ignoring value).",
       UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
