@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -301,11 +301,11 @@ upperlimit_write_comments(struct mkcatalogparams *p,
       gal_list_str_add(comments, str, 0);
     }
 
-  if( asprintf(&str, "Random number generator name: %s", p->rngname)<0 )
+  if( asprintf(&str, "Random number generator name: %s", p->rng_name)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(comments, str, 0);
 
-  if( asprintf(&str, "Random number generator seed: %"PRIu64, p->seed)<0 )
+  if( asprintf(&str, "Random number generator seed: %lu", p->rng_seed)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(comments, str, 0);
 
@@ -372,13 +372,13 @@ upperlimit_write_check(struct mkcatalogparams *p, gal_list_sizet_t *check_x,
 
   /* Put the arrays into a data container. */
   x=gal_data_alloc(xarr, GAL_TYPE_SIZE_T, 1, &num, NULL, 0, p->cp.minmapsize,
-                   "RANDOM_X", "pixel",
+                   p->cp.quietmmap, "RANDOM_X", "pixel",
                    "X-axis position of random footprint's first pixel.");
   y=gal_data_alloc(yarr, GAL_TYPE_SIZE_T, 1, &num, NULL, 0, p->cp.minmapsize,
-                   "RANDOM_Y", "pixel",
+                   p->cp.quietmmap, "RANDOM_Y", "pixel",
                    "Y-axis position of random footprint's first pixel.");
   s=gal_data_alloc(sarr, GAL_TYPE_FLOAT32, 1, &num, NULL, 0, p->cp.minmapsize,
-                   "RANDOM_SUM",
+                   p->cp.quietmmap, "RANDOM_SUM",
                    p->values->unit ? p->values->unit : "input-units",
                    "Sum of pixel values over random footprint.");
 
@@ -506,7 +506,7 @@ upperlimit_measure(struct mkcatalog_passparams *pp, int32_t clumplab,
                      keep the size here also. */
                   init_size=pp->up_vals->size;
                   sum=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0,
-                                     -1, NULL, NULL, NULL);
+                                     -1, 1, NULL, NULL, NULL);
                   ((float *)(sum->array))[0]=o[clumplab?CCOL_SUM:OCOL_SUM];
                   qfunc=gal_statistics_quantile_function(pp->up_vals, sum, 1);
 
@@ -723,7 +723,7 @@ upperlimit_calculate(struct mkcatalog_passparams *pp)
   struct mkcatalogparams *p=pp->p;
 
   /* First find the upper limit magnitude for this object. */
-  upperlimit_one_tile(pp, pp->tile, p->seed+pp->object, 0);
+  upperlimit_one_tile(pp, pp->tile, p->rng_seed+pp->object, 0);
 
   /* If a clumps image is present (a clump catalog is requested) and this
      object has clumps, then find the upper limit magnitude for the clumps
@@ -751,7 +751,7 @@ upperlimit_calculate(struct mkcatalog_passparams *pp)
          IDs. */
       for(i=0;i<pp->clumpsinobj;++i)
         {
-          seed = p->seed + p->numobjects + p->numclumps * pp->object + i;
+          seed = p->rng_seed + p->numobjects + p->numclumps * pp->object + i;
           upperlimit_one_tile(pp, &clumptiles[i], seed, i+1);
         }
 
