@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -62,19 +62,18 @@ oneprofile_center_oversampled(struct mkonthread *mkp)
 {
   struct mkprofparams *p=mkp->p;
 
-  double *dim;
   long os=p->oversample;
+  double *dim, r=1000000;
   size_t i, id=mkp->ibq->id;
   double val, pixfrac, intpart;
 
   for(i=0;i<p->ndim;++i)
     {
       dim = i==0 ? p->x : p->y;
-
       pixfrac = modf(fabs(dim[id]), &intpart);
       val     = ( os*(mkp->width[i]/2 + pixfrac)
                   + (pixfrac<0.5f ? os/2 : -1*os/2-1) );
-      mkp->center[i] = round(val*100)/100;
+      mkp->center[i] = round(val*r)/r;
     }
 }
 
@@ -515,7 +514,7 @@ oneprofile_ispsf(uint8_t fcode)
 
 /* Prepare all the parameters for any type of profile. */
 void
-oneprof_set_prof_params(struct mkonthread *mkp)
+oneprofile_set_prof_params(struct mkonthread *mkp)
 {
   struct mkprofparams *p=mkp->p;
 
@@ -639,10 +638,12 @@ oneprof_set_prof_params(struct mkonthread *mkp)
       mkp->correction       = 0;
       break;
 
+
+
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can correct "
-            "this problem. The profile code %u is not recognized.", __func__,
-            mkp->func);
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can "
+            "correct this problem. The profile code %u is not recognized.",
+            __func__, mkp->func);
     }
 }
 
@@ -694,8 +695,8 @@ oneprofile_make(struct mkonthread *mkp)
 
   /* Allocate and clear the array for this one profile. */
   mkp->ibq->image=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, ndim, dsize,
-                                 NULL, 1, p->cp.minmapsize, "MOCK",
-                                 "Brightness", NULL);
+                                 NULL, 1, p->cp.minmapsize, p->cp.quietmmap,
+                                 "MOCK", "Brightness", NULL);
 
 
   /* Build the profile in the image. */
