@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -37,6 +37,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/checkset.h>
+#include <gnuastro-internal/tile-internal.h>
 
 #include "main.h"
 
@@ -189,7 +190,7 @@ threshold_apply(struct noisechiselparams *p, float *value1,
 void
 threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
                          gal_data_t *inind, char *filename,
-                         gal_list_str_t *comments)
+                         gal_list_str_t *comments, char *extname)
 {
   gal_data_t *sn, *ind, *cols;
 
@@ -202,12 +203,6 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
       sn=gal_data_copy(insn);
       gal_blank_remove(ind);
       gal_blank_remove(sn);
-
-      /* A small sanity check. */
-      if(ind->size==0 || sn->size==0)
-        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-              "the problem. For some reason, all the elements in `ind' or "
-              "`sn' are blank", __func__, PACKAGE_BUGREPORT);
     }
   else
     {
@@ -224,9 +219,11 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
   gal_table_comments_add_intro(&comments, PROGRAM_STRING, &p->rawtime);
 
 
-  /* write the table. */
-  gal_checkset_writable_remove(filename, 0, 1);
-  gal_table_write(cols, comments, p->cp.tableformat, filename, "SN");
+  /* Write the table. Note that we'll set the `dontdelete' argument to 0
+     because when the output is a FITS table, we want all the tables in one
+     FITS file. We have already deleted any existing file with the same
+     name in `ui_set_output_names'.*/
+  gal_table_write(cols, comments, p->cp.tableformat, filename, extname, 0);
 
 
   /* Clean up (if necessary). */
@@ -279,8 +276,9 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
   /* Do the interpolation of both arrays. */
   (*first)->next = *second;
   if(third) (*second)->next = *third;
-  tmp=gal_interpolate_close_neighbors(*first, tl, cp->interpnumngb,
-                                      cp->numthreads, cp->interponlyblank, 1);
+  tmp=gal_interpolate_close_neighbors(*first, tl, cp->interpmetric,
+                                      cp->interpnumngb, cp->numthreads,
+                                      cp->interponlyblank, 1);
   gal_data_free(*first);
   gal_data_free(*second);
   if(third) gal_data_free(*third);
@@ -297,12 +295,13 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
       (*first)->name="THRESH1_INTERP";
       (*second)->name="THRESH2_INTERP";
       if(third) (*third)->name="THRESH3_INTERP";
-      gal_tile_full_values_write(*first, tl, 1, filename, NULL, PROGRAM_NAME);
-      gal_tile_full_values_write(*second, tl, 1, filename, NULL,
-                                 PROGRAM_NAME);
+      gal_tile_full_values_write(*first, tl, !p->ignoreblankintiles,
+                                 filename, NULL, PROGRAM_NAME);
+      gal_tile_full_values_write(*second, tl, !p->ignoreblankintiles,
+                                 filename, NULL, PROGRAM_NAME);
       if(third)
-        gal_tile_full_values_write(*third, tl, 1, filename, NULL,
-                                   PROGRAM_NAME);
+        gal_tile_full_values_write(*third, tl, !p->ignoreblankintiles,
+                                   filename, NULL, PROGRAM_NAME);
       (*first)->name = (*second)->name = NULL;
       if(third) (*third)->name=NULL;
     }
@@ -337,13 +336,13 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
           (*first)->name="THRESH1_SMOOTH";
           (*second)->name="THRESH2_SMOOTH";
           if(third) (*third)->name="THRESH3_SMOOTH";
-          gal_tile_full_values_write(*first, tl, 1, filename, NULL,
-                                     PROGRAM_NAME);
-          gal_tile_full_values_write(*second, tl, 1, filename, NULL,
-                                     PROGRAM_NAME);
+          gal_tile_full_values_write(*first, tl, !p->ignoreblankintiles,
+                                     filename, NULL, PROGRAM_NAME);
+          gal_tile_full_values_write(*second, tl, !p->ignoreblankintiles,
+                                     filename, NULL, PROGRAM_NAME);
           if(third)
-            gal_tile_full_values_write(*third, tl, 1, filename, NULL,
-                                       PROGRAM_NAME);
+            gal_tile_full_values_write(*third, tl, !p->ignoreblankintiles,
+                                       filename, NULL, PROGRAM_NAME);
           (*first)->name = (*second)->name = NULL;
           if(third) (*third)->name=NULL;
         }
@@ -392,19 +391,18 @@ qthresh_on_tile(void *in_prm)
   struct qthreshparams *qprm=(struct qthreshparams *)tprm->params;
   struct noisechiselparams *p=qprm->p;
 
-  double *darr;
   void *tarray=NULL;
   int type=qprm->erode_th->type;
-  gal_data_t *modeconv = p->wconv ? p->wconv : p->conv;
-  gal_data_t *tile, *mode, *qvalue, *usage, *tblock=NULL;
+  gal_data_t *meanconv = p->wconv ? p->wconv : p->conv;
   size_t i, tind, twidth=gal_type_sizeof(type), ndim=p->input->ndim;
+  gal_data_t *tile, *mean, *num, *meanquant, *qvalue, *usage, *tblock=NULL;
 
   /* Put the temporary usage space for this thread into a data set for easy
      processing. */
   usage=gal_data_alloc(gal_pointer_increment(qprm->usage,
                                              tprm->id*p->maxtcontig, type),
                        type, ndim, p->maxtsize, NULL, 0, p->cp.minmapsize,
-                       NULL, NULL, NULL);
+                       p->cp.quietmmap, NULL, NULL, NULL);
 
   /* Go over all the tiles given to this thread. */
   for(i=0; tprm->indexs[i] != GAL_BLANK_SIZE_T; ++i)
@@ -425,31 +423,36 @@ qthresh_on_tile(void *in_prm)
          the convolved image, then copy the desired contents into the
          already allocated `usage' array. */
       tarray=tile->array; tblock=tile->block;
-      tile->array=gal_tile_block_relative_to_other(tile, modeconv);
-      tile->block=modeconv;
+      tile->array=gal_tile_block_relative_to_other(tile, meanconv);
+      tile->block=meanconv;
       gal_data_copy_to_allocated(tile, usage);
-      tile->array=tarray; tile->block=tblock;
+      tile->array=tarray;
+      tile->block=tblock;
 
 
-      /* Find the mode on this tile, note that we have set the `inplace'
-         flag to `1' to avoid extra allocation. */
-      mode=gal_statistics_mode(usage, p->mirrordist, 1);
+      /* Find the mean's quantile on this tile, note that we have already
+         copied the tile's dataset to a newly allocated place. So we have
+         set the `inplace' flag to `1' to avoid extra allocation. */
+      mean=gal_statistics_mean(usage);
+      num=gal_statistics_number(usage);
+      mean=gal_data_copy_to_new_type_free(mean, usage->type);
+      meanquant = ( *(size_t *)(num->array)
+                    ? gal_statistics_quantile_function(usage, mean, 1)
+                    : NULL );
 
-
-      /* Check the mode value. Note that if the mode is not accurate, then
-         the contents of `darr' will be NaN and all conditions will
-         fail. In such cases, the tile will be ignored. */
-      darr=mode->array;
-      if( fabs(darr[1]-0.5f) < p->modmedqdiff )
+      /* Only continue if the mean's quantile is close enough to the
+         median.  */
+      if( meanquant
+          && fabs( *(double *)(meanquant->array)-0.5f) < p->meanmedqdiff )
         {
-          /* The mode was found on the wider convolved image, but the
+          /* The mean was found on the wider convolved image, but the
              qthresh values have to be found on the sharper convolved
              images. This is because the distribution becomes more skewed
              with a wider kernel, helping us find tiles with no data more
              easily. But for the quantile threshold, we want to use the
              sharper convolved image to loose less of the spatial
              information. */
-          if(modeconv!=p->conv)
+          if(meanconv!=p->conv)
             {
               tarray=tile->array; tblock=tile->block;
               tile->array=gal_tile_block_relative_to_other(tile, p->conv);
@@ -474,7 +477,7 @@ qthresh_on_tile(void *in_prm)
           gal_data_free(qvalue);
 
           /* Same for the expansion quantile. */
-          if(p->detgrowquant!=1.0f)
+          if(qprm->expand_th)
             {
               qvalue=gal_statistics_quantile(usage, p->detgrowquant, 1);
               memcpy(gal_pointer_increment(qprm->expand_th->array, tind,
@@ -489,13 +492,15 @@ qthresh_on_tile(void *in_prm)
                                                  tind, type), type);
           gal_blank_write(gal_pointer_increment(qprm->noerode_th->array,
                                                  tind, type), type);
-          if(p->detgrowquant!=1.0f)
+          if(qprm->expand_th)
             gal_blank_write(gal_pointer_increment(qprm->expand_th->array,
                                                    tind, type), type);
         }
 
       /* Clean up and fix the tile's pointers. */
-      gal_data_free(mode);
+      gal_data_free(num);
+      gal_data_free(mean);
+      gal_data_free(meanquant);
     }
 
   /* Clean up and wait for the other threads to finish, then return. */
@@ -509,143 +514,59 @@ qthresh_on_tile(void *in_prm)
 
 
 
-/* The main working function for `threshold_qthresh_clean'. The main
-   purpose/problem is this: when we have channels, the qthresh values for
-   each channel should be treated independently. */
 static void
-threshold_qthresh_clean_work(struct noisechiselparams *p, gal_data_t *first,
-                             gal_data_t *second, gal_data_t *third,
-                             size_t start, size_t number)
+threshold_good_error(size_t number, int before0_after1, size_t interpnumngb)
 {
-  gal_data_t *quantile;
-  size_t i, osize=first->size;
-  float *oa1=NULL, *oa2=NULL, *oa3=NULL;
-  float q, *arr1=NULL, *arr2=NULL, *arr3=NULL;
+  before0_after1=1;
 
-  /* A small sanity check. */
-  if(first->type!=GAL_TYPE_FLOAT32)
-    error(EXIT_FAILURE, 0, "%s: datatype has to be float32", __func__);
+  /* Set the differing strings. */
+  char *in1 = ( before0_after1
+                ? "after removing outliers"
+                : "for defining a quantile threshold" );
+  char *in2 = ( before0_after1
+                ? ""
+                : "NOTE that this is happening *BEFORE* outlier rejection "
+                  "(where the number may decrease even further).");
+  char *in3 = ( before0_after1
+                ? "\n"
+                  "  - (slightly) Increase `--outliersclip' to reject less "
+                  "as outliers.\n"
+                  "  - (slightly) Increase `--outliersigma' to reject less "
+                  "as outliers.\n"
+                : "\n");
 
-  /* Correct the arrays (if necessary). IMPORTANT: The datasets are
-     multi-dimensional. However, when estimating the quantile, their
-     dimensionality doesn't matter (only the `size' element is checked by
-     `gal_statistics_quantile', not `ndim' or `dsize'). So we just need to
-     correct `size' if channels are to be considered. */
-  if(start || number!=first->size)
-    {
-      /* Keep the original values for re-setting later. */
-      oa1=first->array;
-      oa2=second->array;
-      if(third) oa3=third->array;
+  /* Print the error message and abort. */
+  error(EXIT_FAILURE, 0, "%zu tiles usable %s!\n\n"
 
-      /* Increment the array pointers. */
-      first->array=gal_pointer_increment(first->array, start, first->type);
-      second->array=gal_pointer_increment(second->array, start,
-                                           second->type);
-      if(third)
-        third->array=gal_pointer_increment(third->array, start, third->type);
+        "This is smaller than the requested minimum value of %zu (value to "
+        "the `--interpnumngb' option). %s\n\n"
 
-      /* Correct their sizes. */
-      first->size=number;
-      second->size=number;
-      if(third) third->size=number;
-    }
+        "There are several ways to address the problem. The best and most "
+        "highly recommended is to use a larger input if possible (when the "
+        "input is a crop from a larger dataset). If this is not the case, "
+        "or it doesn't solve the problem, you need to loosen the "
+        "parameters mentioned below in the respective order (and therefore "
+        "cause scatter/inaccuracy in the final result). Hence its best to "
+        "not loosen them too much (recall that you can see all the option "
+        "values to Gnuastro's programs by appending `-P' to the end of your "
+        "command).\n"
+        "  - (slightly) Decrease `--tilesize' so your tile-grid has more "
+        "tiles.\n"
+        "  - (slightly) Increase `--meanmedqdiff' to accept more tiles.%s"
+        "  - (slightly) Decrease `--interpnumngb' to be less than %zu.\n\n"
 
-  /* Find the quantile and remove all tiles that are more than it in the
-     first array. */
-  arr1=first->array;
-  quantile=gal_statistics_quantile(first, p->qthreshtilequant, 0);
-  q=*((float *)(quantile->array));
-  for(i=0;i<first->size;++i)
-    /* Just note that we have blank (NaN) values, so to avoid doing a
-       NaN check with `isnan', we will check if the value is below the
-       quantile, if it succeeds (isn't NaN and is below the quantile),
-       then we'll put it's actual value, otherwise, a NaN. */
-    arr1[i] = arr1[i]<q ? arr1[i] : NAN;
-  gal_data_free(quantile);
-
-  /* Second quantile threshold. */
-  arr2=second->array;
-  quantile=gal_statistics_quantile(second, p->qthreshtilequant, 0);
-  q=*((float *)(quantile->array));
-  for(i=0;i<second->size;++i)
-    arr2[i] = arr2[i]<q ? arr2[i] : NAN;
-  gal_data_free(quantile);
-
-  /* The third (if it exists). */
-  if(third)
-    {
-      arr3=third->array;
-      quantile=gal_statistics_quantile(third, p->qthreshtilequant, 0);
-      q=*((float *)(quantile->array));
-      for(i=0;i<third->size;++i)
-        arr3[i] = arr3[i]<q ? arr3[i] : NAN;
-      gal_data_free(quantile);
-    }
-
-  /* Make sure all three have the same NaN pixels. */
-  for(i=0;i<first->size;++i)
-    if( isnan(arr1[i]) || isnan(arr2[i]) || isnan(arr3[i]) )
-      {
-        arr1[i] = arr2[i] = NAN;
-        if(third) arr3[i] = NAN;
-      }
-
-  /* Correct the values if they were changed. */
-  if(start || number!=osize)
-    {
-      first->array=oa1;
-      second->array=oa2;
-      first->size = second->size = osize;
-      if(third) { third->array=oa3; third->size=osize; }
-    }
-}
-
-
-
-
-
-/* Clean higher valued quantile thresholds: useful when the diffuse (almost
-   flat) structures are much larger than the tile size. */
-static void
-threshold_qthresh_clean(struct noisechiselparams *p, gal_data_t *first,
-                        gal_data_t *second, gal_data_t *third,
-                        char *filename)
-{
-  size_t i;
-  struct gal_tile_two_layer_params *tl=&p->cp.tl;
-
-  /* A small sanity check: */
-  if(first->size!=tl->tottiles)
-    error(EXIT_FAILURE, 0, "%s: `first->size' and `tl->tottiles' must have "
-          "the same value, but they don't: %zu, %zu", __func__, first->size,
-          tl->tottiles);
-
-  /* If the input is from a tile structure and the user has asked to ignore
-     channels, then re-order the values. */
-  for(i=0;i<tl->totchannels;++i)
-    threshold_qthresh_clean_work(p, first, second, third,
-                                 i*tl->tottilesinch, tl->tottilesinch);
-
-  /* If the user wants to see the steps. */
-  if(p->qthreshname)
-    {
-      first->name="QTHRESH_ERODE_CLEAN";
-      second->name="QTHRESH_NOERODE_CLEAN";
-      gal_tile_full_values_write(first, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
-      gal_tile_full_values_write(second, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
-      first->name=second->name=NULL;
-
-      if(third)
-        {
-          third->name="QTHRESH_EXPAND_CLEAN";
-          gal_tile_full_values_write(third, tl, 1, p->qthreshname,
-                                     NULL, PROGRAM_NAME);
-          third->name=NULL;
-        }
-    }
+        "---- Tip ----\n"
+        "Append your command with `--checkqthresh' to see the "
+        "successful tiles in relation with this dataset's contents "
+        "before this crash. A visual inspection will greatly help in "
+        "finding the cause/solution for this particular dataset (note "
+        "that the output of `--checkqthresh' is a multi-extension FITS "
+        "file).\n\n"
+        "To better understand this important step, please run the "
+        "following command (press `SPACE'/arrow-keys to navigate and "
+        "`Q' to return back to the command-line):\n\n"
+        "    $ info gnuastro \"Quantifying signal in a tile\"\n", number,
+        in1, interpnumngb, in2, in3, number);
 }
 
 
@@ -685,16 +606,16 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
   /* Allocate space for the quantile threshold values. */
   qprm.erode_th=gal_data_alloc(NULL, p->input->type, p->input->ndim,
                                tl->numtiles, NULL, 0, cp->minmapsize,
-                               NULL, p->input->unit, NULL);
+                               p->cp.quietmmap, NULL, p->input->unit, NULL);
   qprm.noerode_th=gal_data_alloc(NULL, p->input->type, p->input->ndim,
                                  tl->numtiles, NULL, 0, cp->minmapsize,
-                                 NULL, p->input->unit, NULL);
-  if(p->detgrowquant!=1.0f)
-    qprm.expand_th=gal_data_alloc(NULL, p->input->type, p->input->ndim,
-                                  tl->numtiles, NULL, 0, cp->minmapsize,
-                                  NULL, p->input->unit, NULL);
-  else
-    qprm.expand_th=NULL;
+                                 p->cp.quietmmap, NULL, p->input->unit, NULL);
+  qprm.expand_th = ( p->detgrowquant!=1.0f
+                     ? gal_data_alloc(NULL, p->input->type, p->input->ndim,
+                                      tl->numtiles, NULL, 0, cp->minmapsize,
+                                      p->cp.quietmmap, NULL, p->input->unit,
+                                      NULL)
+                     : NULL );
 
 
   /* Allocate temporary space for processing in each tile. */
@@ -716,32 +637,35 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
       if(qprm.expand_th) qprm.expand_th->flag  |= GAL_DATA_FLAG_HASBLANK;
     }
   qprm.noerode_th->flag |= GAL_DATA_FLAG_BLANK_CH;
-  if(p->detgrowquant!=1.0f) qprm.expand_th->flag  |= GAL_DATA_FLAG_BLANK_CH;
+  if(qprm.expand_th) qprm.expand_th->flag  |= GAL_DATA_FLAG_BLANK_CH;
   if(p->qthreshname)
     {
       qprm.erode_th->name="QTHRESH_ERODE";
       qprm.noerode_th->name="QTHRESH_NOERODE";
-      gal_tile_full_values_write(qprm.erode_th, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
-      gal_tile_full_values_write(qprm.noerode_th, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
+      gal_tile_full_values_write(qprm.erode_th, tl,
+                                 !p->ignoreblankintiles,
+                                 p->qthreshname, NULL, PROGRAM_NAME);
+      gal_tile_full_values_write(qprm.noerode_th, tl,
+                                 !p->ignoreblankintiles,
+                                 p->qthreshname, NULL, PROGRAM_NAME);
       qprm.erode_th->name=qprm.noerode_th->name=NULL;
 
       if(qprm.expand_th)
         {
           qprm.expand_th->name="QTHRESH_EXPAND";
-          gal_tile_full_values_write(qprm.expand_th, tl, 1, p->qthreshname,
-                                     NULL, PROGRAM_NAME);
+          gal_tile_full_values_write(qprm.expand_th, tl,
+                                     !p->ignoreblankintiles,
+                                     p->qthreshname, NULL, PROGRAM_NAME);
           qprm.expand_th->name=NULL;
         }
     }
 
 
-  /* Remove higher thresholds if requested. */
-  if(p->qthreshtilequant!=1.0)
-    threshold_qthresh_clean(p, qprm.erode_th, qprm.noerode_th,
-                            qprm.expand_th ? qprm.expand_th : NULL,
-                            p->qthreshname);
+  /* Remove outliers if requested. */
+  if(p->outliersigma!=0.0)
+    gal_tileinternal_no_outlier(qprm.erode_th, qprm.noerode_th,
+                                qprm.expand_th, &p->cp.tl, p->outliersclip,
+                                p->outliersigma, p->qthreshname);
 
 
   /* Check if the number of acceptable tiles is more than the minimum
@@ -750,30 +674,8 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
      errors in parallel. */
   num=gal_statistics_number(qprm.erode_th);
   nval=((size_t *)(num->array))[0];
-  if( nval < cp->interpnumngb)
-    error(EXIT_FAILURE, 0, "%zu tiles can be used for interpolation of the "
-          "quantile threshold values over the full dataset. This is smaller "
-          "than the requested minimum value of %zu (value to the "
-          "`--interpnumngb' option).\n\n"
-          "There are several ways to address the problem. The best and most "
-          "highly recommended is to use a larger input if possible (when the "
-          "input is a crop from a larger dataset). If that is not the case, "
-          "or it doesn't solve the problem, you need to loosen the "
-          "parameters (and therefore cause scatter in the final result). "
-          "Thus don't loosen them too much. Recall that you can see all the "
-          "option values to Gnuastro's programs by appending `-P' to the "
-          "end of your command.\n\n"
-          "  * Decrease `--interpnumngb' to be smaller than %zu.\n"
-          "  * Slightly decrease `--tilesize' to have more tiles.\n"
-          "  * Slightly increase `--modmedqdiff' to accept more tiles.\n\n"
-          "Try appending your command with `--checkqthresh' to see the "
-          "successful tiles (and get a feeling of the cause/solution. Note "
-          "that the output is a multi-extension FITS file).\n\n"
-          "To better understand this important step, please run the "
-          "following command (press `SPACE'/arrow-keys to navigate and "
-          "`Q' to return back to the command-line):\n\n"
-          "    $ info gnuastro \"Quantifying signal in a tile\"\n",
-          nval, cp->interpnumngb, cp->interpnumngb);
+  if( nval < cp->interpnumngb )
+    threshold_good_error(nval, 1, cp->interpnumngb);
 
 
   /* Interpolate and smooth the derived values. */
