@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018, Free Software Foundation, Inc.
+Copyright (C) 2018-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -118,6 +118,7 @@ ui_initialize_options(struct segmentparams *p,
   p->medstd              = NAN;
   p->minstd              = NAN;
   p->maxstd              = NAN;
+  p->snquant             = NAN;
   p->clumpsnthresh       = NAN;
 
   /* Modify common options. */
@@ -126,10 +127,15 @@ ui_initialize_options(struct segmentparams *p,
       /* Select individually. */
       switch(cp->coptions[i].key)
         {
+        case GAL_OPTIONS_KEY_HDU:
+          cp->coptions[i].doc="HDU containing values (science image).";
+          break;
+
         case GAL_OPTIONS_KEY_LOG:
         case GAL_OPTIONS_KEY_TYPE:
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
+        case GAL_OPTIONS_KEY_STDINTIMEOUT:
           cp->coptions[i].flags=OPTION_HIDDEN;
           break;
 
@@ -266,6 +272,21 @@ ui_read_check_only_options(struct segmentparams *p)
               "(starting from zero), extension name, or anything "
               "acceptable by CFITSIO");
     }
+
+  /* If the S/N quantile is less than 0.1 (an arbitrary small value), this
+     is probably due to forgetting that this is the purity level
+     (higher-is-better), not the contamination level
+     (lower-is-better). This actually happened in a few cases: where we
+     wanted a false detection rate of 0.0001 (a super-high value!), and
+     instead of inputing 0.9999, we mistakenly gave `--snquant' a value of
+     `0.0001'. We were thus fully confused with the output (an extremely
+     low value) and thought its a bug, while it wasn't! */
+  if(p->snquant<0.1)
+    fprintf(stderr, "\nWARNING: Value of `--snquant' (`-c') is %g. Note "
+            "that this is not a contamination rate (where lower is "
+            "better), it is a purity rate (where higher is better). If you "
+            "intentionally asked for such a low purity level, please "
+            "ignore this warning\n\n", p->snquant);
 }
 
 
@@ -364,10 +385,10 @@ ui_set_output_names(struct segmentparams *p)
     {
       p->clumpsn_s_name=gal_checkset_automatic_output(&p->cp, basename,
                  ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                   ? "_clumpsn_sky.txt" : "_clumpsn_sky.fits") );
+                   ? "_clumpsn_sky.txt" : "_clumpsn.fits") );
       p->clumpsn_d_name=gal_checkset_automatic_output(&p->cp, basename,
                  ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                   ? "_clumpsn_det.txt" : "_clumpsn_det.fits") );
+                   ? "_clumpsn_det.txt" : "_clumpsn.fits") );
     }
 
   /* Segmentation steps. */
@@ -384,14 +405,19 @@ static void
 ui_prepare_inputs(struct segmentparams *p)
 {
   int32_t *i, *ii;
-  gal_data_t *maxd, *ccin, *ccout=NULL;
+  gal_data_t *maxd, *ccin, *blankflag, *ccout=NULL;
 
   /* Read the input as a single precision floating point dataset. */
   p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
-                                           GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           NULL, GAL_TYPE_FLOAT32,
+                                           p->cp.minmapsize, p->cp.quietmmap);
   p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
                                &p->input->nwcs);
+  p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                            p->input->dsize,
+                                            p->input->wcs);
+
+  /* Set the name. */
   if(p->input->name) free(p->input->name);
   gal_checkset_allocate_copy("INPUT", &p->input->name);
 
@@ -412,8 +438,12 @@ ui_prepare_inputs(struct segmentparams *p)
     {
       /* Read the input convolved image. */
       p->conv = gal_array_read_one_ch_to_type(p->convolvedname, p->chdu,
-                                              GAL_TYPE_FLOAT32,
-                                              p->cp.minmapsize);
+                                              NULL, GAL_TYPE_FLOAT32,
+                                              p->cp.minmapsize,
+                                              p->cp.quietmmap);
+      p->conv->ndim=gal_dimension_remove_extra(p->conv->ndim,
+                                               p->conv->dsize,
+                                               p->conv->wcs);
       p->conv->wcs=gal_wcs_copy(p->input->wcs);
 
       /* Make sure it is the same size as the input. */
@@ -431,7 +461,10 @@ ui_prepare_inputs(struct segmentparams *p)
     {
       /* Read the dataset into memory. */
       p->olabel = gal_array_read_one_ch(p->useddetectionname, p->dhdu,
-                                        p->cp.minmapsize);
+                                        NULL, p->cp.minmapsize,
+                                        p->cp.quietmmap);
+      p->olabel->ndim=gal_dimension_remove_extra(p->olabel->ndim,
+                                                 p->olabel->dsize, NULL);
       if( gal_dimension_is_different(p->input, p->olabel) )
         error(EXIT_FAILURE, 0, "`%s' (hdu: %s) and `%s' (hdu: %s) have a"
               "different dimension/size", p->useddetectionname, p->dhdu,
@@ -450,6 +483,16 @@ ui_prepare_inputs(struct segmentparams *p)
               p->dhdu, gal_type_name(p->olabel->type, 1),
               p->useddetectionname, p->dhdu);
 
+      /* If the input has blank values, set them to blank values in the
+         labeled image too. It doesn't matter if the labeled image has
+         blank pixels that aren't blank on the input image. */
+      if(gal_blank_present(p->input, 1))
+        {
+          blankflag=gal_blank_flag(p->input);
+          gal_blank_flag_apply(p->olabel, blankflag);
+          gal_data_free(blankflag);
+        }
+
       /* Get the maximum value of the input (total number of labels if they
          are separate). If the maximum is 1 (the image is a binary image),
          then apply the connected components algorithm to separate the
@@ -481,7 +524,8 @@ ui_prepare_inputs(struct segmentparams *p)
       /* Allocate the array. */
       p->olabel=gal_data_alloc(NULL, GAL_TYPE_INT32, p->input->ndim,
                                p->input->dsize, p->input->wcs, 0,
-                               p->cp.minmapsize, NULL, NULL, NULL);
+                               p->cp.minmapsize, p->cp.quietmmap,
+                               NULL, NULL, NULL);
 
       /* Initialize it to 1. */
       ii=(i=p->olabel->array)+p->olabel->size; do *i++=1; while(i<ii);
@@ -506,8 +550,14 @@ ui_prepare_kernel(struct segmentparams *p)
   if(p->kernelname)
     {
       if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
-        p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                           p->cp.minmapsize);
+        {
+          p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
+                                             p->cp.minmapsize,
+                                             p->cp.quietmmap);
+          p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
+                                                     p->kernel->dsize,
+                                                     NULL);
+        }
       else
         p->kernel=NULL;
     }
@@ -517,7 +567,7 @@ ui_prepare_kernel(struct segmentparams *p)
          allocated array. */
       p->kernel=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 2,
                                kernel_2d_dsize, NULL, 0, p->cp.minmapsize,
-                               NULL, NULL, NULL);
+                               p->cp.quietmmap, NULL, NULL, NULL);
 
       /* Now copy the staticly allocated array into it. */
       k=p->kernel->array;
@@ -689,7 +739,7 @@ ui_read_std_and_sky(struct segmentparams *p)
     {
       /* Allocate the dataset to keep the value and write it in. */
       p->std=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0,
-                            -1, NULL, NULL, NULL);
+                            -1, 1, NULL, NULL, NULL);
       *(float *)(p->std->array) = tmpval;
     }
   else
@@ -706,8 +756,10 @@ ui_read_std_and_sky(struct segmentparams *p)
 
       /* Read the STD image. */
       p->std=gal_array_read_one_ch_to_type(p->usedstdname, p->stdhdu,
-                                           GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           NULL, GAL_TYPE_FLOAT32,
+                                           p->cp.minmapsize, p->cp.quietmmap);
+      p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
+                                              p->std->dsize, NULL);
 
       /* Make sure it has the correct size. */
       ui_check_size(p->input, p->std, tl->tottiles, p->inputname, p->cp.hdu,
@@ -748,7 +800,7 @@ ui_read_std_and_sky(struct segmentparams *p)
       if(*tailptr=='\0')
         {
           sky=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                             NULL, NULL, NULL);
+                             1, NULL, NULL, NULL);
           *(float *)(sky->array) = skyval = tmpval;
         }
       else
@@ -764,8 +816,10 @@ ui_read_std_and_sky(struct segmentparams *p)
 
           /* Read the Sky dataset. */
           sky=gal_array_read_one_ch_to_type(p->skyname, p->skyhdu,
-                                            GAL_TYPE_FLOAT32,
-                                            p->cp.minmapsize);
+                                            NULL, GAL_TYPE_FLOAT32,
+                                            p->cp.minmapsize, p->cp.quietmmap);
+          sky->ndim=gal_dimension_remove_extra(sky->ndim, sky->dsize,
+                                               NULL);
 
           /* Check its size. */
           ui_check_size(p->input, sky, tl->tottiles, p->inputname, p->cp.hdu,
@@ -880,6 +934,10 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct segmentparams *p)
   gal_options_print_state(&p->cp);
 
 
+  /* Prepare all the options as FITS keywords to write in output later. */
+  gal_options_as_fits_keywords(&p->cp);
+
+
   /* Check that the options and arguments fit well with each other. Note
      that arguments don't go in a configuration file. So this test should
      be done after (possibly) printing the option values. */
@@ -894,7 +952,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct segmentparams *p)
   if(!p->cp.quiet)
     {
       /* Basic inputs. */
-      printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
       printf("  - Using %zu CPU thread%s\n", p->cp.numthreads,
              p->cp.numthreads==1 ? "." : "s.");
       printf("  - Input: %s (hdu: %s)\n", p->inputname, p->cp.hdu);
