@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -27,6 +27,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <error.h>
 #include <stdio.h>
 
+#include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/tile.h>
@@ -132,7 +133,7 @@ ui_initialize_options(struct statisticsparams *p,
   p->quantmax            = NAN;
   p->mirror              = NAN;
   p->mirrordist          = NAN;
-  p->modmedqdiff         = NAN;
+  p->meanmedqdiff        = NAN;
   p->sclipparams[0]      = NAN;
   p->sclipparams[1]      = NAN;
 
@@ -248,9 +249,11 @@ ui_add_to_single_value(struct argp_option *option, char *arg,
     {
       /* Read the string of numbers. */
       inputs=gal_options_parse_list_of_numbers(arg, filename, lineno);
-      d=inputs->array;
+      if(inputs->size==0)
+        error(EXIT_FAILURE, 0, "`--%s' needs a value", option->name);
 
       /* Do the appropriate operations with the  */
+      d=inputs->array;
       switch(option->key)
         {
         case UI_KEY_QUANTILE:
@@ -416,17 +419,17 @@ ui_read_check_only_options(struct statisticsparams *p)
   if( p->sky )
     {
       /* Mandatory options. */
-      if ( isnan(p->modmedqdiff) || isnan(p->sclipparams[0])
-           || p->cp.interpnumngb==0 || isnan(p->mirrordist) )
-        error(EXIT_FAILURE, 0, "`--modmedqdiff', `--sclipparams', "
-              "`--mirrordist', and `--interpnumngb' are mandatory with "
-              "`--sky'");
+      if( isnan(p->meanmedqdiff) || isnan(p->sclipparams[0])
+          || p->cp.interpmetric==0 || p->cp.interpnumngb==0 )
+        error(EXIT_FAILURE, 0, "`--meanmedqdiff', `--sclipparams', "
+              "`--interpmetric' and `--interpnumngb' are mandatory when "
+              "requesting Sky measurement (`--sky')");
 
       /* If mode and median distance is a reasonable value. */
-      if(p->modmedqdiff>0.5)
-        error(EXIT_FAILURE, 0, "%f not acceptable for `--modmedqdiff'. It "
+      if(p->meanmedqdiff>0.5)
+        error(EXIT_FAILURE, 0, "%f not acceptable for `--meanmedqdiff'. It "
               "cannot take values larger than 0.5 (quantile of median)",
-              p->modmedqdiff);
+              p->meanmedqdiff);
 
       /* If a kernel name has been given, we need the HDU. */
       if(p->kernelname && gal_fits_name_is_fits(p->kernelname)
@@ -459,6 +462,18 @@ ui_read_check_only_options(struct statisticsparams *p)
           error(EXIT_FAILURE, 0, "`--mirrordist' is required for the "
                 "mode-related single measurements (`--mode', `--modequant', "
                 "`--modesym', and `--modesymvalue')");
+        break;
+      case UI_KEY_SIGCLIPSTD:
+      case UI_KEY_SIGCLIPMEAN:
+      case UI_KEY_SIGCLIPNUMBER:
+      case UI_KEY_SIGCLIPMEDIAN:
+        if( isnan(p->sclipparams[0]) )
+          error(EXIT_FAILURE, 0, "`--sclipparams' is necessary with "
+                "sigma-clipping measurements.\n\n"
+                "`--sclipparams' takes two values (separated by a comma) for "
+                "defining the sigma-clip: the multiple of sigma, and tolerance "
+                "(<1) or number of clips (>1).");
+        break;
       }
 
 
@@ -508,8 +523,6 @@ ui_read_check_only_options(struct statisticsparams *p)
 static void
 ui_check_options_and_arguments(struct statisticsparams *p)
 {
-  char *name=NULL;
-
   if(p->inputname)
     {
       /* If input is FITS. */
@@ -523,39 +536,15 @@ ui_check_options_and_arguments(struct statisticsparams *p)
                   "(starting from zero), extension name, or anything "
                   "acceptable by CFITSIO");
 
-          /* If its a table, make sure a column is also specified. */
+          /* If its an image, make sure column isn't given (in case the
+             user confuses an image with a table). */
           p->hdu_type=gal_fits_hdu_format(p->inputname, p->cp.hdu);
-          if(p->hdu_type==IMAGE_HDU)
-            {
-              if(p->column)
-                error(EXIT_FAILURE, 0, "%s (hdu: %s): is a FITS image "
-                      "extension. The `--column' option is only applicable "
-                      "to tables.", p->inputname, p->cp.hdu);
-            }
-          else if(p->column==NULL)
-            if( asprintf(&name, "%s (hdu: %s)", p->inputname, p->cp.hdu)<0 )
-              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+          if(p->hdu_type==IMAGE_HDU && p->column)
+            error(EXIT_FAILURE, 0, "%s (hdu: %s): is a FITS image "
+                  "extension. The `--column' option is only applicable "
+                  "to tables.", p->inputname, p->cp.hdu);
         }
-
-      /* If its not FITS, it must be a table. */
-      else
-        {
-          if(p->column==NULL) name=p->inputname;
-        }
-
-      /* If a column was necessary, but not given, print an error. */
-      if(name)
-        error(EXIT_FAILURE, 0, "%s is a table but no column is "
-              "specified. Please use the `--column' (`-c') option to "
-              "specify a column.\n\nYou can either give it the column number "
-              "(couting from 1), or a match/search in its meta-data (e.g., "
-              "column names). For more information, please run the "
-              "following command (press the `SPACE' key to go down and "
-              "`q' to return to the command-line):\n\n"
-              "    $ info gnuastro \"Selecting table columns\"\n", name);
     }
-  else
-    error(EXIT_FAILURE, 0, "no input file is specified");
 }
 
 
@@ -619,10 +608,10 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
      less-than  */
   if(!isnan(p->greaterequal))
     {
-      tmp=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                        NULL, NULL, NULL);
+      tmp=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1, 1,
+                         NULL, NULL, NULL);
       *((float *)(tmp->array)) = p->greaterequal;
-      cond_g=gal_arithmetic(GAL_ARITHMETIC_OP_LT, flags, ref, tmp);
+      cond_g=gal_arithmetic(GAL_ARITHMETIC_OP_LT, 1, flags, ref, tmp);
       gal_data_free(tmp);
     }
 
@@ -630,10 +619,10 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
   /* Same reasoning as above for `p->greaterthan'. */
   if(!isnan(p->lessthan))
     {
-      tmp=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                        NULL, NULL, NULL);
+      tmp=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1, 1,
+                         NULL, NULL, NULL);
       *((float *)(tmp->array)) = p->lessthan;
-      cond_l=gal_arithmetic(GAL_ARITHMETIC_OP_GE, flags, ref, tmp);
+      cond_l=gal_arithmetic(GAL_ARITHMETIC_OP_GE, 1, flags, ref, tmp);
       gal_data_free(tmp);
     }
 
@@ -647,7 +636,7 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
       cond = isnan(p->greaterequal) ? cond_l : cond_g;
       break;
     case 2:
-      cond = gal_arithmetic(GAL_ARITHMETIC_OP_OR, flagsor, cond_l, cond_g);
+      cond = gal_arithmetic(GAL_ARITHMETIC_OP_OR, 1, flagsor, cond_l, cond_g);
       break;
     }
 
@@ -655,14 +644,14 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
   /* Allocate a blank value to mask all pixels that don't satisfy the
      condition. */
   blank=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
-                       0, -1, NULL, NULL, NULL);
+                       0, -1, 1, NULL, NULL, NULL);
   *((float *)(blank->array)) = NAN;
 
 
   /* Set all the pixels that satisfy the condition to blank. Note that a
      blank value will be used in the proper type of the input in the
      `where' operator.*/
-  gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, flagsor, p->input, cond, blank);
+  gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, 1, flagsor, p->input, cond, blank);
 
 
   /* Reset the blank flags so they are checked again if necessary. */
@@ -690,6 +679,10 @@ ui_make_sorted_if_necessary(struct statisticsparams *p)
       case UI_KEY_MEDIAN:
       case UI_KEY_QUANTILE:
       case UI_KEY_QUANTFUNC:
+      case UI_KEY_SIGCLIPSTD:
+      case UI_KEY_SIGCLIPMEAN:
+      case UI_KEY_SIGCLIPNUMBER:
+      case UI_KEY_SIGCLIPMEDIAN:
         is_necessary=1;
         break;
       }
@@ -721,20 +714,69 @@ ui_make_sorted_if_necessary(struct statisticsparams *p)
 void
 ui_read_columns(struct statisticsparams *p)
 {
-  int toomanycols=0;
-  size_t size, counter=0;
-  gal_data_t *cols, *tmp;
+  int toomanycols=0, tformat;
   gal_list_str_t *column=NULL;
+  gal_data_t *cols, *tmp, *cinfo;
+  size_t size, ncols, nrows, counter=0;
+  gal_list_str_t *lines=gal_options_check_stdin(p->inputname,
+                                                p->cp.stdintimeout, "input");
 
-  /* Define the columns that we want, note that they should be added to the
-     list in reverse. */
+  /* If a reference column is also given, add it to the list of columns to
+     read. */
   if(p->refcol)
     gal_list_str_add(&column, p->refcol, 0);
+
+  /* If no column is specified, Statistics will abort and an error will be
+     printed when the table has more than one column. If there is only one
+     column, there is no need to specify any, so Statistics will use it. */
+  if(p->column==NULL)
+    {
+      /* Get the basic table information. */
+      cinfo=gal_table_info(p->inputname, p->cp.hdu, lines, &ncols, &nrows,
+                           &tformat);
+      gal_data_array_free(cinfo, ncols, 1);
+
+      /* See how many columns it has and take the proper action. */
+      switch(ncols)
+        {
+        case 0:
+          error(EXIT_FAILURE, 0, "%s contains no usable information",
+                ( p->inputname
+                  ? gal_checkset_dataset_name(p->inputname, p->cp.hdu)
+                  : "Standard input" ));
+        case 1:
+          gal_checkset_allocate_copy("1", &p->column);
+          break;
+        default:
+          error(EXIT_FAILURE, 0, "%s is a table containing more than one "
+                "column. However, the specific column to work on isn't "
+                "specified.\n\n"
+                "Please use the `--column' (`-c') option to specify a "
+                "column. You can either give it the column number "
+                "(couting from 1), or a match/search in its meta-data (e.g., "
+                "column names).\n\n"
+                "For more information, please run the following command "
+                "(press the `SPACE' key to go down and `q' to return to the "
+                "command-line):\n\n"
+                "    $ info gnuastro \"Selecting table columns\"\n",
+                ( p->inputname
+                  ? gal_checkset_dataset_name(p->inputname, p->cp.hdu)
+                  : "Standard input" ));
+        }
+
+    }
   gal_list_str_add(&column, p->column, 0);
 
   /* Read the desired column(s). */
-  cols=gal_table_read(p->inputname, p->cp.hdu, column, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize, NULL);
+  cols=gal_table_read(p->inputname, p->cp.hdu, lines, column, p->cp.searchin,
+                      p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
+                      NULL);
+  gal_list_str_free(lines, 1);
+
+  /* If the input was from standard input, we can actually write this into
+     it (for future reporting). */
+  if(p->inputname==NULL)
+    gal_checkset_allocate_copy("statistics", &p->inputname);
 
   /* Put the columns into the proper gal_data_t. */
   size=cols->size;
@@ -791,16 +833,25 @@ void
 ui_preparations(struct statisticsparams *p)
 {
   gal_data_t *check;
+  int keepinputdir=p->cp.keepinputdir;
   struct gal_options_common_params *cp=&p->cp;
   struct gal_tile_two_layer_params *tl=&cp->tl;
+  char *checkbasename = p->cp.output ? p->cp.output : p->inputname;
+
+  /* Change `keepinputdir' based on if an output name was given. */
+  p->cp.keepinputdir = p->cp.output ? 1 : 0;
 
   /* Read the input. */
   if(p->isfits && p->hdu_type==IMAGE_HDU)
     {
       p->inputformat=INPUT_FORMAT_IMAGE;
-      p->input=gal_array_read_one_ch(p->inputname, cp->hdu, cp->minmapsize);
+      p->input=gal_array_read_one_ch(p->inputname, cp->hdu, NULL,
+                                     cp->minmapsize, p->cp.quietmmap);
       p->input->wcs=gal_wcs_read(p->inputname, cp->hdu, 0, 0,
                                  &p->input->nwcs);
+      p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                                p->input->dsize,
+                                                p->input->wcs);
     }
   else
     {
@@ -810,8 +861,12 @@ ui_preparations(struct statisticsparams *p)
 
   /* Read the convolution kernel if necessary. */
   if(p->sky && p->kernelname)
-    p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                       cp->minmapsize);
+    {
+      p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
+                                         cp->minmapsize, p->cp.quietmmap);
+      p->kernel->ndim=gal_dimension_remove_extra(p->kernel->ndim,
+                                                 p->kernel->dsize, NULL);
+    }
 
   /* Tile and channel sanity checks and preparations. */
   if(p->ontile || p->sky)
@@ -824,7 +879,7 @@ ui_preparations(struct statisticsparams *p)
       /* Make the tile check image if requested. */
       if(tl->checktiles)
         {
-          tl->tilecheckname=gal_checkset_automatic_output(cp, p->inputname,
+          tl->tilecheckname=gal_checkset_automatic_output(cp, checkbasename,
                                                           "_tiled.fits");
           check=gal_tile_block_check_tiles(tl->tiles);
           if(p->inputformat==INPUT_FORMAT_IMAGE)
@@ -841,7 +896,7 @@ ui_preparations(struct statisticsparams *p)
 
       /* Set the steps image name. */
       if(p->sky && p->checksky)
-        p->checkskyname=gal_checkset_automatic_output(cp, p->inputname,
+        p->checkskyname=gal_checkset_automatic_output(cp, checkbasename,
                                                       "_sky_steps.fits");
     }
 
@@ -875,6 +930,9 @@ ui_preparations(struct statisticsparams *p)
       if( !isnan(p->mirror) )             ++p->numoutfiles;
       if( p->histogram || p->cumulative ) ++p->numoutfiles;
     }
+
+  /* Reset `keepinputdir' to what it originally was. */
+  p->cp.keepinputdir=keepinputdir;
 }
 
 
@@ -949,6 +1007,14 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct statisticsparams *p)
 
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
+
+
+  /* Prepare all the options as FITS keywords to write in output
+     later. Note that in some modes, there is no output file, and
+     `ui_add_to_single_value' isn't yet prepared. */
+  if( (p->singlevalue && p->ontile) || p->sky || p->histogram \
+      || p->cumulative)
+    gal_options_as_fits_keywords(&p->cp);
 }
 
 
