@@ -32,18 +32,22 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Available letters for short options:
 
-   a b d e f g j k l m n p t u v w x y z
-   A B C E G H J L O Q R W X Y
+   a b d e f g j k l m n p t u v x y z
+   A B C E G H J L O Q R X Y
 */
 enum option_keys_enum
 {
   /* With short-option version. */
+  UI_KEY_WCSFILE         = 'w',
+  UI_KEY_WCSHDU          = 'W',
   UI_KEY_COLUMN          = 'c',
   UI_KEY_INFORMATION     = 'i',
   UI_KEY_COLINFOINSTDOUT = 'O',
   UI_KEY_RANGE           = 'r',
   UI_KEY_SORT            = 's',
   UI_KEY_DESCENDING      = 'd',
+  UI_KEY_HEAD            = 'H',
+  UI_KEY_TAIL            = 't',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
