@@ -4,7 +4,7 @@ Functions dealing with general aspects of all Gnuastro.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,26 +34,15 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION                  "0.7"
+#define GAL_CONFIG_VERSION                  "0.10"
 
-#define GAL_CONFIG_HAVE_LIBGIT2             1
+#define GAL_CONFIG_HAVE_LIBGIT2             yes
 
 #define GAL_CONFIG_HAVE_FITS_IS_REENTRANT   @HAVE_FITS_IS_REENTRANT@
 #define GAL_CONFIG_HAVE_WCSLIB_VERSION      1
 
 #define GAL_CONFIG_HAVE_PTHREAD_BARRIER     1
 
-#define GAL_CONFIG_BIN_OP_UINT8             @HAVE_BIN_OP_UINT8@
-#define GAL_CONFIG_BIN_OP_INT8              @HAVE_BIN_OP_INT8@
-#define GAL_CONFIG_BIN_OP_UINT16            @HAVE_BIN_OP_UINT16@
-#define GAL_CONFIG_BIN_OP_INT16             @HAVE_BIN_OP_INT16@
-#define GAL_CONFIG_BIN_OP_UINT32            @HAVE_BIN_OP_UINT32@
-#define GAL_CONFIG_BIN_OP_INT32             @HAVE_BIN_OP_INT32@
-#define GAL_CONFIG_BIN_OP_UINT64            @HAVE_BIN_OP_UINT64@
-#define GAL_CONFIG_BIN_OP_INT64             @HAVE_BIN_OP_INT64@
-#define GAL_CONFIG_BIN_OP_FLOAT32           @HAVE_BIN_OP_FLOAT32@
-#define GAL_CONFIG_BIN_OP_FLOAT64           @HAVE_BIN_OP_FLOAT64@
-
 #define GAL_CONFIG_SIZEOF_LONG              8
 #define GAL_CONFIG_SIZEOF_SIZE_T            8
 
