@@ -179,9 +179,10 @@ gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
                                 float sigclip_param, int inplace, int quiet);
 
 gal_data_t *
-gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t dist, float thresh,
-                                float width, int inplace, int quiet,
-                                size_t *index);
+gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t numprev,
+                                float sigclip_multip, float sigclip_param,
+                                float thresh, size_t numcontig, int inplace,
+                                int quiet, size_t *index);
 
 
 
