@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -61,7 +61,7 @@ gal_statistics_number(gal_data_t *input)
 {
   size_t counter=0, dsize=1;
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* If there is no blank values in the input, then the total number is
      just the size. */
@@ -86,7 +86,7 @@ gal_statistics_minimum(gal_data_t *input)
 {
   size_t dsize=1, n=0;
   gal_data_t *out=gal_data_alloc(NULL, gal_tile_block(input)->type, 1,
-                                 &dsize, NULL, 1, -1, NULL, NULL, NULL);
+                                 &dsize, NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* See if the input actually has any elements. */
   if(input->size)
@@ -116,7 +116,7 @@ gal_statistics_maximum(gal_data_t *input)
 {
   size_t dsize=1, n=0;
   gal_data_t *out=gal_data_alloc(NULL, gal_tile_block(input)->type, 1,
-                                 &dsize, NULL, 1, -1, NULL, NULL, NULL);
+                                 &dsize, NULL, 1, -1, 1, NULL, NULL, NULL);
   /* See if the input actually has any elements. */
   if(input->size)
     {
@@ -145,7 +145,7 @@ gal_statistics_sum(gal_data_t *input)
 {
   size_t dsize=1, n=0;
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* See if the input actually has any elements. */
   if(input->size)
@@ -170,7 +170,7 @@ gal_statistics_mean(gal_data_t *input)
 {
   size_t dsize=1, n=0;
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* See if the input actually has any elements. */
   if(input->size)
@@ -198,7 +198,7 @@ gal_statistics_std(gal_data_t *input)
   size_t dsize=1, n=0;
   double s=0.0f, s2=0.0f;
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* See if the input actually has any elements. */
   if(input->size)
@@ -223,7 +223,7 @@ gal_statistics_mean_std(gal_data_t *input)
   size_t dsize=2, n=0;
   double s=0.0f, s2=0.0f;
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
   /* See if the input actually has any elements. */
   if(input->size)
     /* Parse the input. */
@@ -287,7 +287,7 @@ gal_statistics_median(gal_data_t *input, int inplace)
   size_t dsize=1;
   gal_data_t *nbs=gal_statistics_no_blank_sorted(input, inplace);;
   gal_data_t *out=gal_data_alloc(NULL, nbs->type, 1, &dsize, NULL, 1, -1,
-                                 NULL, NULL, NULL);
+                                 1, NULL, NULL, NULL);
 
   /* Write the median. */
   if(nbs->size)
@@ -349,7 +349,7 @@ gal_statistics_quantile(gal_data_t *input, double quantile, int inplace)
   size_t dsize=1, index;
   gal_data_t *nbs=gal_statistics_no_blank_sorted(input, inplace);
   gal_data_t *out=gal_data_alloc(NULL, nbs->type, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* Only continue processing if there are non-blank elements. */
   if(nbs->size)
@@ -492,7 +492,7 @@ gal_statistics_quantile_function(gal_data_t *input, gal_data_t *value,
   size_t dsize=1;
   gal_data_t *nbs=gal_statistics_no_blank_sorted(input, inplace);
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
   size_t ind=gal_statistics_quantile_function_index(input, value, inplace);
 
   /* Only continue processing if there are non-blank values. */
@@ -846,6 +846,7 @@ mode_golden_section(struct statistics_mode_params *p)
     /* Set the values at the mirror and at `a' (see above). */          \
     mf=a[m];                                                            \
     af=a[ gal_statistics_quantile_index(2*m+1, MODE_SYM_LOW_Q) ];       \
+    if(mf<=af) return 0;                                                \
                                                                         \
     /* This loop is very similar to that of */                          \
     /* `mode_mirror_max_index_diff'. It will find the index where the */\
@@ -883,7 +884,7 @@ mode_golden_section(struct statistics_mode_params *p)
     /*printf("%zu: %f,%f,%f\n", m, (double)af, (double)mf, (double)bf);*/ \
                                                                         \
     /* For a bad result, return 0 (which will not output any mode). */  \
-    return bf==af ? 0 : (bf-mf)/(mf-af);                                \
+    return bf==af ? 0 : (double)(bf-mf)/(double)(mf-af);                \
   }
 static double
 mode_symmetricity(struct statistics_mode_params *p, size_t m, void *b_val)
@@ -951,12 +952,12 @@ gal_statistics_mode(gal_data_t *input, float mirrordist, int inplace)
   size_t dsize=4, mdsize=1;
   struct statistics_mode_params p;
   int type=gal_tile_block(input)->type;
-  gal_data_t *tmptype=gal_data_alloc(NULL, type, 1, &mdsize, NULL, 1, -1,
+  gal_data_t *tmptype=gal_data_alloc(NULL, type, 1, &mdsize, NULL, 1, -1, 1,
                                      NULL, NULL, NULL);
-  gal_data_t *b_val=gal_data_alloc(NULL, type, 1, &mdsize, NULL, 1, -1,
+  gal_data_t *b_val=gal_data_alloc(NULL, type, 1, &mdsize, NULL, 1, -1, 1,
                                    NULL, NULL, NULL);
   gal_data_t *out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
 
 
   /* A small sanity check. */
@@ -1062,7 +1063,7 @@ statistics_make_mirror(gal_data_t *noblank_sorted, size_t index,
 {
   size_t i, dsize = 2*index+1;
   gal_data_t *mirror=gal_data_alloc(NULL, noblank_sorted->type, 1, &dsize,
-                                    NULL, 1, -1, NULL, NULL, NULL);
+                                    NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* Make sure the index is less than or equal to the number of
      elements. */
@@ -1185,7 +1186,7 @@ enum is_sorted_return
 int
 gal_statistics_is_sorted(gal_data_t *input, int updateflags)
 {
-  int out;
+  int out=GAL_BLANK_INT16; /* On some systems, int may be 16-bits wide. */
 
   /* If the flags are already set, don't bother going over the dataset. */
   if( input->flag & GAL_DATA_FLAG_SORT_CH )
@@ -1205,8 +1206,9 @@ gal_statistics_is_sorted(gal_data_t *input, int updateflags)
        increasing. */
     case 1:
       out=STATISTICS_IS_SORTED_INCREASING;
+      break;
 
-    /* Do the check. */
+    /* Do the check when there is more than one element. */
     default:
       switch(input->type)
         {
@@ -1396,14 +1398,12 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
         }
       else noblank=contig;
 
-
-      /* If there are any non-blank elements, make sure the array is
-         sorted. After this step, we won't be dealing with `noblank' any
-         more but with `sorted'. */
+      /* Make sure the array is sorted. After this step, we won't be
+         dealing with `noblank' any more but with `sorted'. */
       if(noblank->size)
         {
           if( gal_statistics_is_sorted(noblank, 1) )
-            sorted=noblank;
+            sorted = inplace ? noblank : gal_data_copy(noblank);
           else
             {
               if(inplace) sorted=noblank;
@@ -1421,10 +1421,28 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
         sorted=noblank;
     }
 
-  /* When the input's size is zero, just return the actual input. */
+  /* Input's size was zero. Note that we cannot simply copy the zero-sized
+     input dataset, we'll have to allocate it here. */
   else
-    sorted=input;
+    sorted = ( inplace
+               ? input
+               : gal_data_alloc(NULL, input->type, 0, NULL, input->wcs, 0,
+                                input->minmapsize, input->quietmmap,
+                                NULL, NULL, NULL) );
 
+  /* Set the blank and sorted flags if the dataset has zero-elements. Even
+     if having blank values or being sorted is not defined on a
+     zero-element dataset, it is up to different functions to choose what
+     they will do with a zero-element dataset. The flags have to be set
+     after this function any way. */
+  if(sorted->size==0)
+    {
+      sorted->flag |= GAL_DATA_FLAG_SORT_CH;
+      sorted->flag |= GAL_DATA_FLAG_BLANK_CH;
+      sorted->flag |= GAL_DATA_FLAG_SORTED_I;
+      sorted->flag &= ~GAL_DATA_FLAG_HASBLANK;
+      sorted->flag &= ~GAL_DATA_FLAG_SORTED_D;
+    }
 
   /* Return final array. */
   return sorted;
@@ -1536,7 +1554,9 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
             {
               tmp=gal_data_copy_to_new_type_free(gal_statistics_maximum(input),
                                                  GAL_TYPE_FLOAT64);
-              max=*((double *)(tmp->array))+1e-6;
+              max=*((double *)(tmp->array));
+
+              /* Clean up. */
               gal_data_free(tmp);
             }
           else max=ra[1];
@@ -1554,15 +1574,17 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
       gal_data_free(tmp);
       tmp=gal_data_copy_to_new_type_free(gal_statistics_maximum(input),
                                          GAL_TYPE_FLOAT64);
-      max=*((double *)(tmp->array)) + 1e-6;
+      max=*((double *)(tmp->array));
+
+      /* Clean up. */
       gal_data_free(tmp);
     }
 
 
   /* Allocate the space for the bins. */
   bins=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numbins, NULL,
-                      0, input->minmapsize, "bin_center", input->unit,
-                      "Center value of each bin.");
+                      0, input->minmapsize, input->quietmmap, "bin_center",
+                      input->unit, "Center value of each bin.");
 
 
   /* Set central bin values. */
@@ -1591,7 +1613,7 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
   printf("onebinstart: %.10f\n", onebinstart);
   printf("binwidth: %g\n", binwidth);
   for(i=0;i<numbins;++i)
-    printf("%zu: %.4f\t(%f, %f)\n", i, b[i], b[i]-hbw, b[i]+hbw);
+    printf("%zu: %.4g\t(%g, %g)\n", i, b[i], b[i]-hbw, b[i]+hbw);
   */
 
   /* Set the status of the bins to regular and return. */
@@ -1612,7 +1634,16 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
 
 #define HISTOGRAM_TYPESET(IT) {                                         \
     IT *a=input->array, *af=a+input->size;                              \
-    do if( *a>=min && *a<max) ++h[ (size_t)( (*a-min)/binwidth ) ];     \
+    do                                                                  \
+      if(*a>=min && *a<=max)                                            \
+        {                                                               \
+          h_i=(*a-min)/binwidth;                                        \
+          /* When `*a' is the largest element (within floating point */ \
+          /* errors), `h_i' can be one element larger than the       */ \
+          /* number of bins. But since its in the dataset, we need   */ \
+          /* to count it. So we'll put it in the last bin.           */ \
+          ++h[ h_i - (h_i==hist->size ? 1 : 0) ];                       \
+        }                                                               \
     while(++a<af);                                                      \
   }
 
@@ -1620,8 +1651,8 @@ gal_data_t *
 gal_statistics_histogram(gal_data_t *input, gal_data_t *bins, int normalize,
                          int maxone)
 {
-  size_t *h;
   float *f, *ff;
+  size_t *h, h_i;
   gal_data_t *hist;
   double *d, min, max, ref=NAN, binwidth;
 
@@ -1647,15 +1678,16 @@ gal_statistics_histogram(gal_data_t *input, gal_data_t *bins, int normalize,
   /* Allocate the histogram (note that we are clearning it so all values
      are zero. */
   hist=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, bins->ndim, bins->dsize,
-                      NULL, 1, input->minmapsize, "hist_number", "counts",
+                      NULL, 1, input->minmapsize, input->quietmmap,
+                      "hist_number", "counts",
                       "Number of data points within each bin.");
 
 
   /* Set the minimum and maximum range of the histogram from the bins. */
   d=bins->array;
   binwidth=d[1]-d[0];
-  max = d[bins->size - 1] + binwidth/2;
-  min = d[0]              - binwidth/2;
+  min = d[ 0      ] - binwidth/2;
+  max = d[ bins->size-1 ] + binwidth/2;
 
 
   /* Go through all the elements and find out which bin they belong to. */
@@ -1787,7 +1819,7 @@ gal_statistics_cfp(gal_data_t *input, gal_data_t *bins, int normalize)
 
   /* Allocate the cumulative frequency plot's necessary space. */
   cfp=gal_data_alloc( NULL, hist->type, bins->ndim, bins->dsize,
-                      NULL, 1, input->minmapsize,
+                      NULL, 1, input->minmapsize, input->quietmmap,
                       ( hist->type==GAL_TYPE_FLOAT32
                         ? "cfp_normalized" : "cfp_number" ),
                       ( hist->type==GAL_TYPE_FLOAT32
@@ -1953,9 +1985,9 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
 
   /* Allocate the necessary spaces. */
   out=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &four, NULL, 0,
-                     input->minmapsize, NULL, NULL, NULL);
+                     input->minmapsize, input->quietmmap, NULL, NULL, NULL);
   median_i=gal_data_alloc(NULL, type, 1, &one, NULL, 0, input->minmapsize,
-                          NULL, NULL, NULL);
+                          input->quietmmap, NULL, NULL, NULL);
 
 
   /* Only continue processing if we have non-blank elements. */
@@ -1983,20 +2015,29 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
       start=nbs->array;
       while(num<maxnum && size)
         {
-          /* Find the median. */
-          statistics_median_in_sorted_no_blank(nbs, median_i->array);
-          median_d=gal_data_copy_to_new_type(median_i, GAL_TYPE_FLOAT64);
-
           /* Find the average and Standard deviation, note that both
              `start' and `size' will be different in the next round. */
           nbs->array = start;
           nbs->size = oldsize = size;
-          meanstd=gal_statistics_mean_std(nbs);
 
-          /* Put the three final values in usable (with a type)
-             pointers. */
-          med  = median_d->array;
+          /* For a detailed check, just correct the type).
+          if(!quiet)
+            {
+              size_t iii;
+              printf("nbs->size: %zu\n", nbs->size);
+              for(iii=0;iii<nbs->size;++iii)
+                printf("%f\n", ((float *)(nbs->array))[iii]);
+            }
+          */
+
+          /* Find the mean, median and standard deviation. */
+          meanstd=gal_statistics_mean_std(nbs);
+          statistics_median_in_sorted_no_blank(nbs, median_i->array);
+          median_d=gal_data_copy_to_new_type(median_i, GAL_TYPE_FLOAT64);
+
+          /* Put them in usable (with a type) pointers. */
           mean = meanstd->array;
+          med  = median_d->array;
           std  = &((double *)(meanstd->array))[1];
 
           /* If the user wanted to view the steps, show it to them. */
@@ -2069,61 +2110,232 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
 
 
 
-#if 0
-/* Using the cumulative distribution function this function will
-   remove outliers from a dataset. */
-void
-gal_statistics_remove_outliers_flat_cdf(float *sorted, size_t *outsize)
+/* Find the first outlier in a distribution. */
+#define OUTLIER_BYTYPE(IT) {                                            \
+    IT *arr=nbs->array;                                                 \
+    for(i=window_size;i<nbs->size;++i)                                  \
+      {                                                                 \
+        /* Fill in the distance array. */                               \
+        for(j=0; j<wtakeone; ++j)                                       \
+          darr[j] = arr[i-window_size+j+1] - arr[i-window_size+j];      \
+                                                                        \
+        /* Get the sigma-clipped information. */                        \
+        sclip=gal_statistics_sigma_clip(dist, sigclip_multip,           \
+                                        sigclip_param, 0, 1);           \
+        sarr=sclip->array;                                              \
+                                                                        \
+        /* For a check */                                               \
+        if(quiet==0)                                                    \
+          printf("%f [%zu]: %f (%f, %f) %f\n", (float)(arr[i]), i,      \
+                 (float)(arr[i]-arr[i-1]), sarr[1], sarr[3],            \
+                 (((double)(arr[i]-arr[i-1])) - sarr[1])/sarr[3]);      \
+                                                                        \
+        /* Terminate the loop if the dist. is larger than requested. */ \
+        /* This shows we have reached the first outlier's position. */  \
+        if( (((double)(arr[i]-arr[i-1])) - sarr[1]) > sigma*sarr[3] )   \
+          {                                                             \
+            /* Allocate the output dataset. */                          \
+            out=gal_data_alloc(NULL, input->type, 1, &one, NULL, 0, -1, \
+                               1, NULL, NULL, NULL);                    \
+                                                                        \
+            /* Write the outlier, clean up and break. */                \
+            *(IT *)(out->array)=arr[i-1];                               \
+            gal_data_free(sclip);                                       \
+            break;                                                      \
+          }                                                             \
+                                                                        \
+        /* Clean up (if we get here). */                                \
+        gal_data_free(sclip);                                           \
+      }                                                                 \
+  }
+gal_data_t *
+gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
+                                float sigma, float sigclip_multip,
+                                float sigclip_param, int inplace, int quiet)
 {
-  printf("\n ... in gal_statistics_remove_outliers_flat_cdf ... \n");
-  exit(1);
-  int firstfound=0;
-  size_t size=*outsize, i, maxind;
-  float *slopes, minslope, maxslope;
+  float *sarr;
+  double *darr;
+  size_t i, j, one=1, wtakeone;
+  gal_data_t *dist, *sclip, *nbs, *out=NULL;
 
-  /* Find a slopes array, think of the cumulative frequency plot when
-     you want to think about slopes. */
-  errno=0; slopes=malloc(size*sizeof *slopes);
-  if(slopes==NULL)
-    error(EXIT_FAILURE, errno, "%s: %zu bytes for slopes",
-          __func__, size*sizeof *slopes);
+  /* Remove all blanks and sort the dataset. */
+  nbs=gal_statistics_no_blank_sorted(input, inplace);
 
-  /* Calcuate the slope of the CDF and put it in the slopes array. */
-  for(i=1;i<size-1;++i)
-    slopes[i]=2/(sorted[i+1]-sorted[i-1]);
+  /* If all elements are blank, simply return the default (NULL) output. */
+  if(nbs->size==0) return out;
 
-  /* Find the position of the maximum slope, note that around the
-     distribution mode, the difference between the values varies less,
-     so two neighbouring elements have the closest values, hence the
-     largest slope (when their difference is in the denominator). */
-  gal_statistics_f_max_with_index(slopes+1, size-2, &maxslope, &maxind);
+  /* Only continue if the window size is more than 2 elements (out
+     "outlier" is hard to define on smaller datasets). */
+  if(window_size>2)
+    {
+      /* For a check.
+      if(nbs->type==GAL_TYPE_FLOAT32)
+        {
+          float *n=nbs->array;
+          for(i=0;i<nbs->size;++i)
+            printf("%f\n", n[i]);
+          exit(0);
+        }
+      */
 
-  /* Find the minimum slope from the second element (for the first the
-     slope is not defined. NOTE; maxind is one smaller than it should
-     be because the input array to find it began from the second
-     element. */
-  gal_statistics_float_second_min(slopes+1, maxind+1, &minslope);
 
-  /* Find the second place where the slope falls below `minslope`
-     after the maximum position. When found, add it with one to
-     account for error. Note that incase there are no outliers by this
-     definition, then the final size will be equal to size! */
-  for(i=maxind+1;i<size-1;++i)
-    if(slopes[i]<minslope)
-      {
-        if(firstfound)
-          break;
-        else
-          firstfound=1;
-      }
-  *outsize=i+1;
+      /* Allocate space to keep the distances. */
+      wtakeone=window_size-1;
+      dist=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &wtakeone, NULL,
+                          0, -1, 1, NULL, NULL, NULL);
+      darr=dist->array;
 
-  /*
-  for(i=0;i<size;++i)
-    printf("%zu\t%.3f\t%.3f\n", i, arr[i], slopes[i]);
-  printf("\n\nPlace to cut off for outliers is: %zu\n\n", *outsize);
-  */
+      /* Find the outlier based on the type of the input dataset. */
+      switch(input->type)
+        {
+        case GAL_TYPE_UINT8:     OUTLIER_BYTYPE( uint8_t  );   break;
+        case GAL_TYPE_INT8:      OUTLIER_BYTYPE( int8_t   );   break;
+        case GAL_TYPE_UINT16:    OUTLIER_BYTYPE( uint16_t );   break;
+        case GAL_TYPE_INT16:     OUTLIER_BYTYPE( int16_t  );   break;
+        case GAL_TYPE_UINT32:    OUTLIER_BYTYPE( uint32_t );   break;
+        case GAL_TYPE_INT32:     OUTLIER_BYTYPE( int32_t  );   break;
+        case GAL_TYPE_UINT64:    OUTLIER_BYTYPE( uint64_t );   break;
+        case GAL_TYPE_INT64:     OUTLIER_BYTYPE( int64_t  );   break;
+        case GAL_TYPE_FLOAT32:   OUTLIER_BYTYPE( float    );   break;
+        case GAL_TYPE_FLOAT64:   OUTLIER_BYTYPE( double   );   break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+                __func__, input->type);
+        }
 
-  free(slopes);
+      /* Clean up. */
+      gal_data_free(dist);
+    }
+
+  /* Clean up and return. */
+  if(nbs!=input) gal_data_free(nbs);
+  return out;
+}
+
+
+
+
+
+
+/* Find the outliers using the average distance of the neighboring
+   points. */
+#define OUTLIER_FLAT_CFP_BYTYPE(IT) {                                   \
+    IT diff, *pr=prev->array;                                           \
+    IT *a=nbs->array, *p=a+d, *pp=a+nbs->size-d;                        \
+                                                                        \
+    do                                                                  \
+      {                                                                 \
+        diff=*(p+d)-*(p-d);                                             \
+        if(p-a-d<numprev)                                               \
+          {                                                             \
+            pr[p-a-d]=diff;                                             \
+            if(!quiet) printf("%-6zu%-15g%-15g\n", p-a, (float)(*p),    \
+                              (float)diff);                             \
+          }                                                             \
+        else                                                            \
+          {                                                             \
+            /* Sigma-clipped median and std for a check. */             \
+            prev->flag=0;                                               \
+            prev->size=prev->dsize[0]=numprev;                          \
+            sclip=gal_statistics_sigma_clip(prev, sigclip_multip,       \
+                                            sigclip_param, 1, 1);       \
+                                                                        \
+            sarr=sclip->array;                                          \
+            check = (diff - sarr[1]) / sarr[3];                         \
+                                                                        \
+            /* If requested, print the values. */                       \
+            if(!quiet) printf("%-6zu%-15g%-15g%-15g (%g,%g)\n", p-a,    \
+                              (float)(*p), (float)diff, check, sarr[1], \
+                              sarr[3]);                                 \
+                                                                        \
+            /* When values are equal, std will be roughly zero */       \
+            if(sarr[3]>1e-6 && check>thresh)                            \
+              {                                                         \
+                if(flatind==GAL_BLANK_SIZE_T)                           \
+                  {                                                     \
+                    ++counter;                                          \
+                    flatind=p-a;                                        \
+                  }                                                     \
+                else                                                    \
+                  {                                                     \
+                    if(flatind==p-a-counter)                            \
+                      { /* First element above thresh is 0, so for */   \
+                        /* counting, when counting the number of */     \
+                        /* contiguous elements, we have to add 1. */    \
+                        if(counter+1==numcontig)                        \
+                          {gal_data_free(sclip); break;}                \
+                        else ++counter;                                 \
+                      }                                                 \
+                    else { flatind=GAL_BLANK_SIZE_T; counter=0; }       \
+                  }                                                     \
+              }                                                         \
+            else { flatind=GAL_BLANK_SIZE_T; counter=0; }               \
+            pr[(p-a-d)%numprev]=diff;                                   \
+            gal_data_free(sclip);                                       \
+          }                                                             \
+      }                                                                 \
+    while(++p<pp);                                                      \
+    if(counter+1!=numcontig) flatind=GAL_BLANK_SIZE_T;                    \
+  }
+
+gal_data_t *
+gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t numprev,
+                                float sigclip_multip, float sigclip_param,
+                                float thresh, size_t numcontig, int inplace,
+                                int quiet, size_t *index)
+{
+  float *sarr;
+  double check;
+  gal_data_t  *nbs, *prev, *out=NULL, *sclip;
+  size_t d=2, counter=0, one=1, flatind=GAL_BLANK_SIZE_T;
+
+  /* Sanity checks. */
+  if(thresh<=0)
+    error(EXIT_FAILURE, 0, "%s: the value of `thresh' (%g) must be "
+          "positive", __func__, thresh);
+  if(numprev==0)
+    error(EXIT_FAILURE, 0, "%s: `numprev' (%zu) cannot be zero", __func__,
+          numprev);
+
+  /* Remove all blanks and sort the dataset. */
+  nbs=gal_statistics_no_blank_sorted(input, inplace);
+
+  /* Keep previous slopes. */
+  prev=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numprev, NULL, 0, -1,
+                      1, NULL, NULL, NULL);
+
+  /* Find the index where the distribution becomes sufficiently flat. */
+  switch(nbs->type)
+    {
+    case GAL_TYPE_UINT8:   OUTLIER_FLAT_CFP_BYTYPE( uint8_t  ); break;
+    case GAL_TYPE_INT8:    OUTLIER_FLAT_CFP_BYTYPE( int8_t   ); break;
+    case GAL_TYPE_UINT16:  OUTLIER_FLAT_CFP_BYTYPE( uint16_t ); break;
+    case GAL_TYPE_INT16:   OUTLIER_FLAT_CFP_BYTYPE( int16_t  ); break;
+    case GAL_TYPE_UINT32:  OUTLIER_FLAT_CFP_BYTYPE( uint32_t ); break;
+    case GAL_TYPE_INT32:   OUTLIER_FLAT_CFP_BYTYPE( int32_t  ); break;
+    case GAL_TYPE_UINT64:  OUTLIER_FLAT_CFP_BYTYPE( uint64_t ); break;
+    case GAL_TYPE_INT64:   OUTLIER_FLAT_CFP_BYTYPE( int64_t  ); break;
+    case GAL_TYPE_FLOAT32: OUTLIER_FLAT_CFP_BYTYPE( float    ); break;
+    case GAL_TYPE_FLOAT64: OUTLIER_FLAT_CFP_BYTYPE( double   ); break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+            __func__, nbs->type);
+    }
+
+  /* Write the output dataset: if no point flat part was found, return
+     NULL. */
+  if(flatind!=GAL_BLANK_SIZE_T)
+    {
+      out=gal_data_alloc(NULL, input->type, 1, &one, NULL, 0, -1, 1,
+                         NULL, NULL, NULL);
+      memcpy(out->array,
+             gal_pointer_increment(nbs->array, flatind, nbs->type),
+             gal_type_sizeof(nbs->type));
+    }
+
+  /* Clean up and return. */
+  if(nbs!=input) gal_data_free(nbs);
+  if(index) *index=flatind;
+  gal_data_free(prev);
+  return out;
 }
-#endif
