@@ -410,7 +410,7 @@ gal_table_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
 gal_data_t *
 gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
                gal_list_str_t *cols, int searchin, int ignorecase,
-               size_t minmapsize, size_t *colmatch)
+               size_t minmapsize, int quietmmap, size_t *colmatch)
 {
   int tableformat;
   gal_list_sizet_t *indexll;
@@ -442,13 +442,13 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
     {
     case GAL_TABLE_FORMAT_TXT:
       out=gal_txt_table_read(filename, lines, numrows, allcols, indexll,
-                             minmapsize);
+                             minmapsize, quietmmap);
       break;
 
     case GAL_TABLE_FORMAT_AFITS:
     case GAL_TABLE_FORMAT_BFITS:
       out=gal_fits_tab_read(filename, hdu, numrows, allcols, indexll,
-                            minmapsize);
+                            minmapsize, quietmmap);
       break;
 
     default:
