@@ -6,6 +6,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2019 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -53,7 +54,12 @@ if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
 # The number of threads is one so if CFITSIO does is not configured to
 # enable multithreaded access to files, the tests pass. It is the
 # users choice to enable this feature.
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
 cat=$topsrc/tests/$prog/cat.txt
-$execname $img --catalog=$cat --suffix=_imgcat.fits --numthreads=1      \
-          --zeroisnotblank --coordcol=X_CENTER --coordcol=Y_CENTER      \
-          --namecol=NAME --mode=img --width=201
+$check_with_program $execname $img --catalog=$cat --suffix=_imgcat.fits    \
+                              --numthreads=1 --zeroisnotblank --mode=img   \
+                              --coordcol=X_CENTER --coordcol=Y_CENTER      \
+                              --namecol=NAME  --width=201
